local AddonName, AddonTable = ...
local L = AddonTable.Localize
LibStub:GetLibrary("BigLibTimer3"):Register(AddonTable)

if not SuperDuperSoundPackEngineAddonConfig then
	SuperDuperSoundPackEngineAddonConfig = {}
end

local SoundPacks = {}

function Register_SuperDuperSoundPackEngine_SoundPack(Name)
	if type(Name) == "string" and Name ~= "" then
		if type(SoundPacks[Name]) ~= "table" then
			SoundPacks[Name] = {}
		end
		return SoundPacks[Name]
	end
	return nil
end

local DefaultSoundPack = "SuperMarioWorldOfWarcraft"

local function SoundPackValue(Key)
	return SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack][Key]
end


local CLASS = select(2, UnitClass("player"))
local RACE = select(2, UnitRace("player")):upper():gsub("[^A-Z]", ""):gsub("SCOURGE", "UNDEAD")

local VARIABLES_CHECKED = nil
local SOUND_SUSPENDED = nil
local CHECKS_DISABLED = nil

local LOADING = true
local SoundPacksLoaded = nil
local COMBAT = nil
local STARMUSIC = nil
local SWIMMINGMUSIC = nil
local FLYINGMUSIC = nil
local VEHICLEMUSIC = nil
local RESTINGMUSIC = nil
local MOUNTEDMUSIC = nil
local SPEEDMUISIC = nil
local MUSIC_DELAYED = nil
local MOUNTED = nil
local GAMEOVER = nil
local BATTLEFIELDWINNER = nil
local WASINPVEINSTANCE = nil
local LAST_HIT_ENEMY_PLAYER_TARGET_GUID = nil
local LAST_ENEMY_PLAYER_TARGET_GUID = nil
local BOSSID = {}
local SHIELDBUFFS = {
	(GetSpellInfo(17--[[Power Word: Shield]])),
	(GetSpellInfo(642--[[Divine Shield]])),
	(GetSpellInfo(1463--[[Mana Shield]])),
	(GetSpellInfo(11426--[[Ice Barrier]])),
	(GetSpellInfo(7812--[[Sacrifice]])),
}
local FORMBUFFS = {
	(GetSpellInfo(1066--[[Aquatic Form]])),
	(GetSpellInfo(5487--[[Bear Form]])),
	(GetSpellInfo(768--[[Cat Form]])),
	(GetSpellInfo(9634--[[Dire Bear Form]])),
	(GetSpellInfo(33943--[[Flight Form]])),
	(GetSpellInfo(24858--[[Moonkin Form]])),
	(GetSpellInfo(40120--[[Swift Flight Form]])),
	(GetSpellInfo(783--[[Travel Form]])),
	(GetSpellInfo(65139--[[Tree of Life]])),
	(GetSpellInfo(2645--[[Ghost Wolf]])),
	(GetSpellInfo(59672--[[Metamorphosis]])),
}

local SHIELDBUFFS_SIZE = select("#", unpack(SHIELDBUFFS))
local function IsShield(buff)
	if buff then
		for i = 1, SHIELDBUFFS_SIZE do
			if buff == SHIELDBUFFS[i] then
				return true
			end
		end
	else
		for i = 1, SHIELDBUFFS_SIZE do
			if SHIELDBUFFS[i] and UnitBuff("player", SHIELDBUFFS[i]) then
				STARMUSIC = 1
				return true
			end
		end
		STARMUSIC = nil
	end
	return false
end

local FORMBUFFS_SIZE = select("#", unpack(FORMBUFFS))
local function IsForm(buff)
	if CLASS == "DRUID" or CLASS == "WARLOCK" or CLASS == "SHAMAN" then
		if buff then
			for i = 1, FORMBUFFS_SIZE do
				if buff == FORMBUFFS[i] then
					return true
				end
			end
		elseif GetShapeshiftForm() ~= 0 then
			return true
		end
	end
	return false
end


local function ContinueSound(...)
	local File = ...
	local count = select("#", ...)
	if count > 1 then
		File = select(math.random(1, count), ...)
	end
	if type(File) == "string" then
		PlaySoundFile(File)
	end
end

function AddonTable.SOUND(...)
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED then
		local First = ...
		if type(First) == "table" then
			ContinueSound(unpack(First))
		else
			ContinueSound(...)
		end
	end
end


local function ContinueMusic(...)
	local File = ...
	local count = select("#", ...)
	if count > 1 then
		local number = math.random(1, count)
		File = select(number, ...)
		if type(File) ~= "string" or LAST_MULTI_MUSIC_FILE_USED == File then
			File = {...}
			tremove(File, number)
			File = select(math.random(1, count - 1), unpack(File))
		end
		LAST_MULTI_MUSIC_FILE_USED = File
		AddonTable:SetTimer("ResetMUSIC", 300, 300, AddonTable.MUSIC, ...)
	else
		AddonTable:ClearTimer("ResetMUSIC")
	end
	if type(File) == "string" then
		PlayMusic(File)
	end
end

local LAST_MULTI_MUSIC_FILE_USED = nil

function AddonTable.MUSIC(...)
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
		local First = ...
		if type(First) == "table" then
			ContinueMusic(unpack(First))
		else
			ContinueMusic(...)
		end
	end
end


local function SilenceMusic()
	AddonTable:ClearTimer("ResetMUSIC")
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED then
		PlayMusic("Interface\\AddOns\\"..AddonName.."\\silence.ogg")
	end
end


local function SetMusic()
	if AddonTable:GetTimer("Delay") <= 0 or not MUSIC_DELAYED then
		MUSIC_DELAYED = nil
		if UnitIsDead("player") and not UnitIsGhost("player") then
			SilenceMusic()
			AddonTable:ClearTimer("star")
		elseif AddonTable:IsTimer("star") and STARMUSIC and not SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED then
			SilenceMusic()
		elseif AddonTable:GetTimer("PLAYER_ENTERING_WORLD") <= 0 then
			if STARMUSIC and not SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED then
				SilenceMusic()
				if not AddonTable:IsTimer("star") then
					AddonTable:SetTimer("star", nil, 3.15, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\star.mp3")
				end
			else
				AddonTable:ClearTimer("star")
				if SPEEDMUISIC then
					AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Speed_Mario_2.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Speed_Mario_Techno_Remix.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Speed_smb.mp3")
				elseif not COMBAT and SWIMMINGMUSIC then
					AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Underwater.mp3")
				elseif not COMBAT and FLYINGMUSIC then
					AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Coin_Heaven_1.mp3")
				elseif not COMBAT and VEHICLEMUSIC then
					AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Kart_Circuit.mp3")
				else
					if select(2, IsInInstance()) == "party" or select(2, IsInInstance()) == "raid" then
						if COMBAT then
							AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Castle_1.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Castle_king_coopa.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Castle_Remix_SMB.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Castle_supermario.mp3")
						else
							AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Dungeon_1.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Dungeon_Underworld_SMB3.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Dungeon_undworld_funk_sm3.mp3")
						end
					elseif COMBAT then
						AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Battle_mb.mp3")
					elseif RESTINGMUSIC then
						AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\sm64_title.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\smas_selectgame.mp3")
					elseif MOUNTEDMUSIC then
						AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Mounted_smw_ovr1_2.mp3")
					else
						AddonTable.MUSIC("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Overworld_da_smb3.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Overworld_Remix_SMB.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Overworld_smb.mp3", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Music\\Overworld_Super_Mario_Brothers_Tropical_Mix.mp3")
					end
				end
			end
		else
			SilenceMusic()
			AddonTable:ClearTimer("star")
		end
	else
		SilenceMusic()
	end
end


local function StopAllMusic(DELAY)
	if DELAY == 0 then return end
	SilenceMusic()
	AddonTable:ClearTimer("star")
	if DELAY then
		MUSIC_DELAYED = 1
		AddonTable:SetTimer("Delay", DELAY, nil, SetMusic)
	else
		MUSIC_DELAYED = nil
		AddonTable:ClearTimer("Delay")
	end
end

local SetMusicVariables = {
	"ENABLE_SOUND",
	"ENABLE_MUSIC",
	"ENABLE_MUSIC_LOOPING",
}
local DelayMusicVariables = {
	"Sound_NumChannels",
	"Sound_OutputDriverIndex",
	"Sound_OutputQuality",
	"ENABLE_REVERB",
	"ENABLE_SOFTWARE_HRTF",
	"ENABLE_DSP_EFFECTS",
	"ENABLE_HARDWARE",
}
hooksecurefunc("SetCVar", function(cvar, value, scriptCVar)
	for _, v in ipairs(SetMusicVariables) do
		if cvar == v or scriptCVar == v then
			if value == "1" then
				SetMusic()
			end
			return
		end
	end
	for _, v in ipairs(DelayMusicVariables) do
		if cvar == v or scriptCVar == v then
			StopAllMusic(math.max(3, AddonTable:GetTimer("Delay")))
			return
		end
	end
end)


local function GroupWiped()
	if not GAMEOVER then
		local group
		local members
		if UnitInRaid("player") then
			members = GetNumRaidMembers()
			group = "raid"
		else
			if not UnitIsDeadOrGhost("player") then
				return
			end
			members = GetNumPartyMembers()
			group = "party"
		end
		for i = 1, members do
			if not UnitIsDeadOrGhost(group..i) and UnitIsConnected(group..i) then
				return
			end
		end
		GAMEOVER = 1
		AddonTable:SetTimer("GroupWiped-GameOver"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb_gameover.ogg")
		StopAllMusic((3.5 or 0) + AddonTable:GetTimer("Delay"))
	end
end


local function TestGameOver()
	if not GAMEOVER and ( UnitInRaid("player") or GetNumPartyMembers() > 0 or CHECKS_DISABLED ) then
		AddonTable:SetTimer("TestGameOver-UNIT_DIED", 0.5, nil, GroupWiped)
	end
end



local function TestPower()
	if UnitCanAttack("player", "target") and CheckInteractDistance("target", 3) and ( not UnitIsDeadOrGhost("target") or UnitExists("targettarget") or UnitAffectingCombat("target") ) and (
		GetComboPoints("player") == 5
		or
		(
			UnitPowerType("player") == SPELL_POWER_RAGE
			and UnitPowerMax("player", SPELL_POWER_RAGE) > 0
			and UnitPower("player", SPELL_POWER_RAGE) == UnitPowerMax("player", SPELL_POWER_RAGE)
		)
	) then
		if not SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED then
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_pmeter.ogg")
		end
	else
		AddonTable:ReplaceTimer("FULLPOWER", nil, 0, 0)
	end
end

local function CheckVariables()
	if not SuperDuperSoundPackEngineAddonConfig then
		SuperDuperSoundPackEngineAddonConfig = {}
	end
	VARIABLES_CHECKED = 1
end


local function LoadOptionsFrameSettings()
	SuperDuperSoundPackEngineAddonOptionsFrame_AllSoundsEnabled:SetChecked(not SOUND_SUSPENDED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Music:SetChecked(not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Sounds:SetChecked(not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Jumping:SetChecked(not SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Swimming:SetChecked(not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Dmg:SetChecked(not SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Coin:SetChecked(not SuperDuperSoundPackEngineAddonConfig.COIN_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_PowerUpOnBuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.POWERUP_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_PowerDownOnDebuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.POWERDOWN_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_RunningOutOfTime:SetChecked(not SuperDuperSoundPackEngineAddonConfig.OUTOFTIME_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_FullPower:SetChecked(not SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_DamageImmunityBuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrame_Checks:SetChecked(not CHECKS_DISABLED)
end


local function ResetDefaults()
	wipe(SuperDuperSoundPackEngineAddonConfig)
	SOUND_SUSPENDED = nil
	CHECKS_DISABLED = nil
end


local function SlashHandler(msg)
	local msg = msg:lower()
	if msg:match("v") then
		print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version"))
	elseif msg:match("check") then
		if CHECKS_DISABLED then
			wipe(BOSSID)
			CHECKS_DISABLED = nil
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..":  Checks enabled.")
		else
			CHECKS_DISABLED = true
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..":  Checks disabled for this session!")
		end
	elseif msg:match("sou") or msg:match("aud") or msg:match("mus") then
		if SOUND_SUSPENDED then
			SOUND_SUSPENDED = nil
			SetMusic()
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..":  Sound enabled.")
		else
			SOUND_SUSPENDED = true
			StopMusic()
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..":  Sound disabled for this session!")
		end
	elseif msg:match("reset") or msg:match("clear") or msg:match("delete") or msg:match("erase") then
		if msg:match("all") then
			SuperDuperSoundPackEngineAddonConfig = nil
			SOUND_SUSPENDED = nil
			CHECKS_DISABLED = nil
			CheckVariables()
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..": Settings have been reset for all players!")
		else
			ResetDefaults()
			print(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version")..": Settings have been reset for "..UnitName("player").."!")
		end
	else
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(AddonName, "Title"))
		AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_pause.ogg")
		return
	end
	LoadOptionsFrameSettings()
	AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_message.ogg")
end
SlashCmdList.SuperDuperSoundPackEngineAddon = SlashHandler
SLASH_SuperDuperSoundPackEngineAddon1 = "/soundpack"
SLASH_SuperDuperSoundPackEngineAddon2 = "/superduper"
SLASH_SuperDuperSoundPackEngineAddon3 = "/sdspe"
SLASH_SuperDuperSoundPackEngineAddon4 = "/supermariowow"
SLASH_SuperDuperSoundPackEngineAddon5 = "/mariowow"
SLASH_SuperDuperSoundPackEngineAddon6 = "/smwow"
SLASH_SuperDuperSoundPackEngineAddon7 = "/mario"
SLASH_SuperDuperSoundPackEngineAddon8 = "/smw"



function OnUpdate()
	if not LOADING then
		if IsMounted() then
			if not MOUNTED then
				MOUNTED = 1
				AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_yoshi.ogg")
			end
		elseif MOUNTED then
			MOUNTED = nil
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\yoshihit.ogg")
		end
		if IsSwimming() then
			if not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED and GetUnitSpeed("player") > 0 and HasFullControl() then
				if not AddonTable:IsTimer("swim") then
					AddonTable:SetTimer("swim", 0, 0.6, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_swim.ogg")
				end
			else
				AddonTable:ClearTimer("swim")
			end
			if not SWIMMINGMUSIC and not SPEEDMUISIC and not COMBAT and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
				SWIMMINGMUSIC = 1
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				SetMusic()
			end
			if FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				if not SWIMMINGMUSIC and not SPEEDMUISIC and not COMBAT then
					SetMusic()
				end
			end
		else
			AddonTable:ClearTimer("swim")
			if IsFlying() or UnitOnTaxi("player") then
				if not FLYINGMUSIC and not SPEEDMUISIC and not COMBAT and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					FLYINGMUSIC = 1
					SWIMMINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					if not FLYINGMUSIC and not SPEEDMUISIC and not COMBAT then
						SetMusic()
					end
				end
			elseif UnitInVehicle("player") then
				if not VEHICLEMUSIC and not SPEEDMUISIC and not COMBAT and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					VEHICLEMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					if not VEHICLEMUSIC and not SPEEDMUISIC and not COMBAT then
						SetMusic()
					end
				end
			elseif IsResting() then
				if not RESTINGMUSIC and not SPEEDMUISIC and not COMBAT and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					RESTINGMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					MOUNTEDMUSIC = nil
					if not RESTINGMUSIC and not SPEEDMUISIC and not COMBAT then
						SetMusic()
					end
				end
			elseif IsMounted() then
				if not MOUNTEDMUSIC and not SPEEDMUISIC and not COMBAT and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					MOUNTEDMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					if not MOUNTEDMUSIC and not SPEEDMUISIC and not COMBAT then
						SetMusic()
					end
				end
			elseif SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
				SWIMMINGMUSIC = nil
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				if not SPEEDMUISIC and not COMBAT then
					SetMusic()
				end
			end
		end
	end
end
local UpdateFrame = CreateFrame("Frame")
UpdateFrame.UpdateInterval = 0.05
UpdateFrame.TimeSinceLastUpdate = 0
UpdateFrame:SetScript("OnUpdate", function(self, elapsed)
	self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed
	if self.TimeSinceLastUpdate >= self.UpdateInterval and not self.Running then
		self.Running = true
		self.TimeSinceLastUpdate = 0
		OnUpdate()
		self.Running = false
	end
end)
UpdateFrame:Show()



local function OnEvent(self, event, ...)
	if event == "PLAYER_REGEN_DISABLED" then
			
			COMBAT = 1
			GAMEOVER = nil
			SetMusic()
			
	elseif event == "PLAYER_REGEN_ENABLED" then
			
			COMBAT = nil
			SPEEDMUISIC = nil
			SetMusic()
			
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			
			if select(2, ...) == "PARTY_KILL" and ( select(4, ...) == UnitGUID("player") or select(4, ...) == UnitGUID("pet") ) then
				AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb_stomp.ogg")
			elseif select(2, ...) == "UNIT_DIED" then
				if BOSSID[select(7, ...)] then
					BOSSID[select(7, ...)] = nil
					AddonTable:SetTimer("BossDied"..GetTime(), (3 or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\Flag_Super_Mario_Bros.mp3")
					StopAllMusic((6 or 0) + (3 or 0) + AddonTable:GetTimer("Delay"))
				elseif LAST_HIT_ENEMY_PLAYER_TARGET_GUID == select(7, ...) then
					if AddonTable:GetTimer("Delay") <= 3 then
						AddonTable:SetTimer("player_kill"..GetTime(), (1 or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\sm64_thank_you.ogg")
						if not AddonTable:ReplaceTimer("Delay", (2.5 or 0) + (1 or 0) + AddonTable:GetTimer("Delay")) then
							AddonTable:SetTimer("Delay", (2.5 or 0) + (1 or 0) + AddonTable:GetTimer("Delay"))
						end
					end
				else
					if select(7, ...) == UnitGUID("player") then
						if AddonTable:GetTimer("Delay") <= 3 then
							AddonTable:SetTimer("PlayerDied"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_death.ogg")
							StopAllMusic((3 or 0) + AddonTable:GetTimer("Delay"))
						end
						STARMUSIC = nil
						SPEEDMUISIC = nil
					end
					TestGameOver()
				end
			elseif select(2, ...) == "SPELL_AURA_APPLIED" and select(7, ...) == UnitGUID("player") then
				if select(13, ...) == "BUFF" then
					if IsForm(select(11, ...)) then
						if not LOADING and not AddonTable:IsTimer("powerup") then
							AddonTable:SetTimer("powerup", (1 or 0))
							AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_transform.ogg")
						end
					else
						if not SuperDuperSoundPackEngineAddonConfig.POWERUP_DISABLED and not AddonTable:IsTimer("powerup") then
							AddonTable:SetTimer("powerup", (1 or 0))
							AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_powerup.ogg")
						end
						if not SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED and not STARMUSIC and IsShield(select(11, ...)) then
							STARMUSIC = 1
							if not MUSIC_DELAYED then
								SilenceMusic()
								if not AddonTable:IsTimer("star") then
									AddonTable:SetTimer("star", AddonTable:GetTimer("powerup"), (3.15 or 0), AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\star.mp3")
								end
							elseif AddonTable:GetTimer("Delay") <= AddonTable:GetTimer("powerup") then
								StopAllMusic(AddonTable:GetTimer("powerup"))
							end
						end
					end
				elseif select(13, ...) == "DEBUFF" and not SuperDuperSoundPackEngineAddonConfig.POWERDOWN_DISABLED and not AddonTable:IsTimer("powerdown") then
					AddonTable:SetTimer("powerdown", 1)
					AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_powerdown.ogg")
				end
			elseif select(2, ...) == "SPELL_AURA_REMOVED" and select(13, ...) == "BUFF" and select(7, ...) == UnitGUID("player") and STARMUSIC and IsShield(select(11, ...)) then
				IsShield()
				if not STARMUSIC then
					SetMusic()
				end
			else
				if ( select(4, ...) == UnitGUID("player") or select(4, ...) == UnitGUID("pet") ) and ( ( select(7, ...) == UnitGUID("target") and UnitIsPlayer("target") and UnitFactionGroup("target") ~= UnitFactionGroup("player") ) or ( not UnitExists("target") and select(7, ...) == LAST_ENEMY_PLAYER_TARGET_GUID ) ) then
					LAST_HIT_ENEMY_PLAYER_TARGET_GUID = select(7, ...)
				end
				if not SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED and select(4, ...) == UnitGUID("player") and select(7, ...) == UnitGUID("target") and UnitCanAttack("player", "target") then
					if select(2, ...) == "SPELL_DAMAGE" then
						AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_fireball.ogg")
					elseif select(2, ...) == "SWING_DAMAGE" then
						AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_tail.ogg")
					elseif select(2, ...) == "RANGE_DAMAGE" then
						AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_kick.ogg")
					end
				end
			end
			
	elseif event == "PLAYER_TARGET_CHANGED" then
			
			if UnitIsPlayer("target") and UnitFactionGroup("target") ~= UnitFactionGroup("player") then
				LAST_ENEMY_PLAYER_TARGET_GUID = UnitGUID("target")
			elseif UnitExists("target") then
				LAST_ENEMY_PLAYER_TARGET_GUID = nil
			end
			
	elseif event == "PLAYER_XP_UPDATE" then
			
			if not SuperDuperSoundPackEngineAddonConfig.COIN_DISABLED then
				AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_coin.ogg")
			end
			
	elseif event == "COMBAT_TEXT_UPDATE" then
			
			if not SuperDuperSoundPackEngineAddonConfig.COIN_DISABLED then
				if (select(1, ...) == "HONOR_GAINED" and select(2, ...) > 0) or (select(1, ...) == "FACTION" and select(3, ...) > 0) then
					AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_coin.ogg")
				end
			end
			
	elseif event == "PLAYER_ALIVE" then
			
			AddonTable:SetTimer("PLAYER_ENTERING_WORLD", 2, nil,
				function()
					if LOADING then
						MOUNTED = IsMounted()
						LOADING = nil
					end
					IsShield()
					SetMusic()
				end
			)
			TestGameOver()
			
	elseif event == "PLAYER_ENTERING_WORLD" then
			
			wipe(BOSSID)
			local inInstance, instanceType = IsInInstance()
			if instanceType ~= "party" and instanceType ~= "raid" then
				inInstance = nil
			end
			if LOADING then
				if not SuperDuperSoundPackEngineAddonConfig.SoundPack or not SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack] then
					if SoundPacks[DefaultSoundPack] then
						SuperDuperSoundPackEngineAddonConfig.SoundPack = DefaultSoundPack
					else
						SuperDuperSoundPackEngineAddonConfig.SoundPack = next(SoundPacks)
					end
				end
				if SuperDuperSoundPackEngineAddonConfig.SoundPack then
					SoundPacksLoaded = true
				end
				AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\sm64_memario.ogg")
			elseif not UnitIsDeadOrGhost("player") then
				if instanceType == "pvp" or instanceType == "arena" then
					AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_stagestart.ogg")
				elseif inInstance ~= WASINPVEINSTANCE then
					AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_pipe.ogg")
				end
			end
			SilenceMusic()
			WASINPVEINSTANCE = inInstance
			if BATTLEFIELDWINNER then
				if BATTLEFIELDWINNER == 1 then
					AddonTable:SetTimer("arena_victory"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, SoundPackValue("arena_victory"))
					StopAllMusic((SoundPackValue("arena_victory_time") or 0) + AddonTable:GetTimer("Delay"))
				elseif BATTLEFIELDWINNER == UnitFactionGroup("player") then
					AddonTable:SetTimer("battleground_victory"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_victory.mp3")
					StopAllMusic((5 or 0) + AddonTable:GetTimer("Delay"))
				elseif BATTLEFIELDWINNER ~= 0 then
					AddonTable:SetTimer("lost_battle"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_gameover.mp3")
					StopAllMusic((7 or 0) + AddonTable:GetTimer("Delay"))
				end
				BATTLEFIELDWINNER = nil
			end
			AddonTable:SetTimer("PLAYER_ENTERING_WORLD", 2, nil,
				function()
					if LOADING then
						MOUNTED = IsMounted()
						LOADING = nil
					end
					IsShield()
					SetMusic()
				end
			)
			TestGameOver()
			
	elseif event == "UPDATE_BATTLEFIELD_SCORE" then
			
			if not BATTLEFIELDWINNER and GetBattlefieldWinner() then
				if select(2, IsInInstance()) == "pvp" then
					BATTLEFIELDWINNER = PLAYER_FACTION_GROUP[GetBattlefieldWinner()]
				elseif select(2, IsInInstance()) == "arena" then
					for i=1,GetNumBattlefieldScores() do
						if GetBattlefieldScore(i) == UnitName("player") then
							if select(6, GetBattlefieldScore(i)) == GetBattlefieldWinner() then
								BATTLEFIELDWINNER = 1
							else
								BATTLEFIELDWINNER = 0
							end
							break
						end
					end
				end
			end
			
	elseif event == "ACHIEVEMENT_EARNED" then
			
			AddonTable:SetTimer("ACHIEVEMENT_EARNED"..GetTime(), AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smas-smb3_goal.ogg")
			StopAllMusic((3 or 0) + AddonTable:GetTimer("Delay"))
			
	elseif event == "CHAT_MSG_SKILL" then
			
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_1-up.ogg")
			
	elseif event == "PLAYER_LEVEL_UP" then
			
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_1-up.ogg")
			AddonTable:SetTimer("PLAYER_LEVEL_UP"..GetTime(), (2 or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.SOUND, "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\Win_smb3king.mp3")
			StopAllMusic((7.5 or 0) + (2 or 0) + AddonTable:GetTimer("Delay"))
			
	elseif event == "ADDON_LOADED" then
			
			if select(1, ...) == AddonName then
				CheckVariables()
			end
			
	end
	
	if not AddonTable:ReplaceTimer("FULLPOWER", 0, 0.8, TestPower) then
		AddonTable:SetTimer("FULLPOWER", 0, 0.8, TestPower)
	end
	
	if ( tostring(UnitClassification("target")):lower():match("boss") or ( UnitLevel("target") == -1 and not UnitPlayerControlled("target") ) or ( CHECKS_DISABLED and UnitCanAttack("player", "target") and not UnitPlayerControlled("target") ) ) and not UnitIsDeadOrGhost("target") then
		BOSSID[UnitGUID("target")] = true
		if UnitAffectingCombat("target") then
			if not SPEEDMUISIC and COMBAT and ( ( UnitHealth("target") / UnitHealthMax("target") ) * 100 ) <= 25 and not UnitIsDeadOrGhost("player") then
				if SuperDuperSoundPackEngineAddonConfig.OUTOFTIME_DISABLED then
					SPEEDMUISIC = 1
					SetMusic()
				elseif not AddonTable:IsTimer("Delay") then
					SPEEDMUISIC = 1
					AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb_warning.ogg")
					StopAllMusic((3 or 0))
				end
			end
		end
		if SPEEDMUISIC and ( not COMBAT or not UnitAffectingCombat("target") or ( ( UnitHealth("target") / UnitHealthMax("target") ) * 100 ) > 25 ) then
			SPEEDMUISIC = nil
			SetMusic()
		end
	end
	
end
local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
EventFrame:RegisterEvent("ADDON_LOADED")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:RegisterEvent("PLAYER_ALIVE")
EventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
EventFrame:RegisterEvent("PLAYER_LEVEL_UP")
EventFrame:RegisterEvent("PLAYER_XP_UPDATE")
EventFrame:RegisterEvent("COMBAT_TEXT_UPDATE")
EventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
EventFrame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE")
EventFrame:RegisterEvent("UNIT_COMBO_POINTS")
EventFrame:RegisterEvent("CHAT_MSG_SKILL")
EventFrame:RegisterEvent("ACHIEVEMENT_EARNED")
EventFrame:SetScript("OnEvent", OnEvent)


hooksecurefunc("ToggleGameMenu", function()
	AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_pause.ogg")
end)

hooksecurefunc("JumpOrAscendStart", function()
	if IsSwimming() then
		if not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED and HasFullControl() and not AddonTable:IsTimer("swim") then
			AddonTable:SetTimer("swim", 0.6)
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_swim.ogg")
		end
	elseif not SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED and HasFullControl() and not IsFlying() and not UnitInVehicle("player") and not AddonTable:IsTimer("jump") then
		AddonTable:SetTimer("jump", 0.6)
		if IsMounted() then
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_jump.ogg")
		elseif RACE == "GNOME" then
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb_jumpsmall.ogg")
		elseif ( RACE == "BLOODELF" or ( RACE == "NIGHTELF" and not UnitIsDeadOrGhost("player") ) ) and not IsForm() then
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_jump.ogg", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_jump.ogg", "Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_spinjump.ogg")
		else
			AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smb3_jump.ogg")
		end
	end
end)


local function SaveOptionsFrameSettings()
	if SuperDuperSoundPackEngineAddonOptionsFrame_AllSoundsEnabled:GetChecked() then
		if SOUND_SUSPENDED then
			SOUND_SUSPENDED = nil
			SetMusic()
		end
	elseif not SOUND_SUSPENDED then
		SOUND_SUSPENDED = true
		StopMusic()
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Music:GetChecked() then
		if SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
			SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED = nil
			SetMusic()
		end
	elseif not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
		SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED = true
		StopMusic()
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Sounds:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Jumping:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Swimming:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Dmg:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Coin:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.COIN_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.COIN_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_PowerUpOnBuff:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.POWERUP_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.POWERUP_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_PowerDownOnDebuff:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.POWERDOWN_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.POWERDOWN_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_RunningOutOfTime:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.OUTOFTIME_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.OUTOFTIME_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_FullPower:GetChecked() then
		SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED = true
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_DamageImmunityBuff:GetChecked() then
		if SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED and STARMUSIC and IsShield() then
			SetMusic()
		end
		SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.STAR_DISABLED = true
		if AddonTable:ClearTimer("star") then
			SetMusic()
		end
	end
	if SuperDuperSoundPackEngineAddonOptionsFrame_Checks:GetChecked() then
		if CHECKS_DISABLED then
			wipe(BOSSID)
			CHECKS_DISABLED = nil
		end
	else
		CHECKS_DISABLED = true
	end
end


function SuperDuperSoundPackEngineAddonOptionsFrame_OnLoad(self)
	
	for _, Frame in next, {self:GetChildren()} do
		if Frame:GetObjectType() == "FontString" then
			local text = Frame:GetText()
			if text and text ~= "" then
				Frame:SetText(L[text])
			end
		else
			for _, Frame in next, {Frame:GetRegions()} do
				if Frame:GetObjectType() == "FontString" then
					local text = Frame:GetText()
					if text and text ~= "" then
						Frame:SetText(L[text])
					end
				end
			end
		end
	end
	
	_G[self:GetName().."TitleString"]:SetText(GetAddOnMetadata(AddonName, "Title").." "..GetAddOnMetadata(AddonName, "Version"))	
	
	-- Set the name for the Category for the Panel
	self.name = GetAddOnMetadata(AddonName, "Title")
	
	-- When the player clicks okay, run this function.
	self.okay = function(self)
		SaveOptionsFrameSettings()
		AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_message.ogg")
	end
	
	-- When the player clicks cancel, run this function.
	self.cancel = function(self)
		AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\nsmb_pause.ogg")
	end
	
	-- This is a function that is called when the player presses the Default Button.
	self.default = function(self)
		ResetDefaults()
		AddonTable.SOUND("Interface\\AddOns\\SuperMarioWorldOfWarcraft\\Sounds\\smw_message.ogg")
	end
	
	-- This will run whenever the options frame is loaded or after defaults are loaded.
	self.refresh = function(self)
		LoadOptionsFrameSettings()
	end
	
	-- Add the panel to the Interface Options
	InterfaceOptions_AddCategory(self)
	
end

